"use strict";

var _unitTest = require("@tractor/unit-test");

var _errorHandler = require("@tractor/error-handler");

var _container = require("./container");

/* global describe:true, it:true */
// Test setup:
// Dependencies:
// Under test:
describe('@tractor/dependency-injection - Container:', function () {
  describe('constant:', function () {
    it('should throw an error if something has already been registered by the given name', function () {
      var container = new _container.Container();
      (0, _unitTest.expect)(function () {
        container.constant({
          foo: 'foo'
        });
        container.constant({
          foo: 'bar'
        });
      }).to.throw(_errorHandler.TractorError, 'Cannot register "foo" with DI container, because there is already a constant registered with that name');
    });
  });
  describe('factory:', function () {
    it('should throw an error if it is given something that is not a function', function () {
      (0, _unitTest.expect)(function () {
        var container = new _container.Container();
        container.factory(null);
      }).to.throw(_errorHandler.TractorError, 'Cannot register factory with DI container, because it is not a function');
    });
    it('should throw an error if it is given an anonymous function', function () {
      var container = new _container.Container();
      (0, _unitTest.expect)(function () {
        container.factory(function () {});
      }).to.throw(_errorHandler.TractorError, 'Cannot register factory with DI container, because it is an anonymous function');
    });
    it('should throw an error if something has already been registered by the given name', function () {
      var container = new _container.Container();
      (0, _unitTest.expect)(function () {
        container.factory(function foo() {});
        container.factory(function foo() {});
      }).to.throw(_errorHandler.TractorError, 'Cannot register "foo" with DI container, because there is already a factory registered with that name');
    });
  });
  describe('call:', function () {
    it('should inject dependencies into an unregistered function', function () {
      var container = new _container.Container();
      var arg = {};
      container.constant({
        arg: arg
      });

      function test() {}

      test['@Inject'] = ['arg'];
      var wrap = {
        test: test
      };

      _unitTest.sinon.stub(wrap, 'test');

      container.call(wrap.test);
      (0, _unitTest.expect)(wrap.test).to.have.been.calledWith(arg);
    });
    it('should work if the function has no dependencies', function () {
      var container = new _container.Container();

      function test() {}

      var wrap = {
        test: test
      };

      _unitTest.sinon.stub(wrap, 'test');

      container.call(wrap.test);
      (0, _unitTest.expect)(wrap.test).to.have.been.called();
    });
    it('should append any extra given arguments', function () {
      var container = new _container.Container();
      var arg1 = {};
      container.constant({
        arg1: arg1
      });
      var arg2 = {};

      function test() {}

      test['@Inject'] = ['arg1'];
      var wrap = {
        test: test
      };

      _unitTest.sinon.stub(wrap, 'test');

      container.call(wrap.test, [arg2]);
      (0, _unitTest.expect)(wrap.test).to.have.been.calledWith(arg1, arg2);
    });
  });
  describe('instantiate:', function () {
    it('should instantiate an instance of a function with no dependencies', function () {
      var container = new _container.Container();

      function Car() {}

      container.factory(Car);
      var car = container.instantiate(Car);
      (0, _unitTest.expect)(car).to.be.an.instanceof(Car);
    });
    it('should instantiate an instance of a function with dependencies', function () {
      var container = new _container.Container();
      var config = {};
      container.constant({
        config: config
      });

      function Engine() {}

      container.factory(Engine);

      function Tractor(config, engine) {
        this.config = config;
        this.engine = engine;
      }

      Tractor['@Inject'] = ['config', 'Engine'];
      container.factory(Tractor);
      var tractor = container.instantiate(Tractor);
      (0, _unitTest.expect)(tractor).to.be.an.instanceof(Tractor);
      (0, _unitTest.expect)(tractor.config).to.equal(config);
      (0, _unitTest.expect)(tractor.engine).to.be.an.instanceof(Engine);
    });
    it('should throw an error if a dependency cannot be resolved', function () {
      var container = new _container.Container();

      function Kitchen() {}

      Kitchen['@Inject'] = ['Sink'];
      container.factory(Kitchen);
      (0, _unitTest.expect)(function () {
        container.instantiate(Kitchen);
      }).to.throw(_errorHandler.TractorError, 'Could not find a factory for "Sink"');
    });
  });
});